import * as React from 'react';
import { WebView } from 'react-native-webview';
import s from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { ModalFooter, ModalTitle } from '../themed/ModalParts';
import { ThemedModal } from '../themed/ThemedModal';
export const ChallengeModal = (props) => {
    const { bridge, challengeError } = props;
    const handleCancel = useHandler(() => bridge.resolve(undefined));
    const handleLoading = useHandler((event) => {
        if (/\/success$/.test(event.url)) {
            bridge.resolve(true);
            return false;
        }
        if (/\/failure$/.test(event.url)) {
            bridge.resolve(false);
            return false;
        }
        return true;
    });
    return (React.createElement(ThemedModal, { bridge: bridge, onCancel: handleCancel },
        React.createElement(ModalTitle, null, s.strings.complete_captcha_title),
        React.createElement(WebView, { source: { uri: challengeError.challengeUri }, 
            // Allow the modal background to appear inside the WebView.
            // This is a magic value from the WebView documentation,
            // so don't use the theme - normal colors don't do anything.
            // eslint-disable-next-line react-native/no-color-literals
            style: { backgroundColor: '#00000000' }, onShouldStartLoadWithRequest: handleLoading }),
        React.createElement(ModalFooter, { onPress: handleCancel })));
};
