import * as React from 'react';
import { initializeLogin } from '../../actions/LoginInitActions';
import { setAppConfig } from '../../common/appConfig';
import { Router } from '../navigation/Router';
import { ReduxStore } from '../services/ReduxStore';
import { changeFont } from '../services/ThemeContext';
export function LoginScreen(props) {
    const { appConfig, context, fontDescription = { regularFontFamily: 'System' }, initialLoginId, username } = props;
    const { regularFontFamily, headingFontFamily = regularFontFamily } = fontDescription;
    const { onComplete, onLogEvent = (event, values) => { } } = props;
    // Look up the requested user:
    const initialUserInfo = initialLoginId != null
        ? context.localUsers.find(info => info.loginId === initialLoginId)
        : username != null
            ? context.localUsers.find(info => info.username === username)
            : undefined;
    // Update theme fonts if they are different:
    React.useEffect(() => changeFont(regularFontFamily, headingFontFamily), [
        regularFontFamily,
        headingFontFamily
    ]);
    setAppConfig(appConfig);
    const branding = {
        appId: props.appId,
        appName: props.appName,
        backgroundImage: props.backgroundImage,
        landingSceneText: props.landingScreenText,
        parentButton: props.parentButton,
        primaryLogo: props.primaryLogo,
        primaryLogoCallback: props.primaryLogoCallback
    };
    return (React.createElement(ReduxStore, { imports: {
            accountOptions: props.accountOptions,
            context,
            initialUserInfo,
            initialRoute: props.initialRoute,
            onComplete,
            onLogin: props.onLogin,
            onLogEvent,
            onNotificationPermit: props.onNotificationPermit,
            recoveryKey: props.recoveryLogin,
            skipSecurityAlerts: props.skipSecurityAlerts,
            customPermissionsFunction: props.customPermissionsFunction
        }, initialAction: initializeLogin() },
        React.createElement(Router, { branding: branding })));
}
