var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeChallengeError, asMaybeOtpError, asMaybePasswordError, asMaybeUsernameError } from 'edge-core-js';
import * as React from 'react';
import { Keyboard, TouchableOpacity, View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { cacheStyles } from 'react-native-patina';
import Animated, { Easing, Extrapolate, interpolate, useAnimatedScrollHandler, useAnimatedStyle, useDerivedValue, useSharedValue, withTiming } from 'react-native-reanimated';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import { sprintf } from 'sprintf-js';
import { launchPasswordRecovery, login } from '../../actions/LoginAction';
import s from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useLocalUsers } from '../../hooks/useLocalUsers';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { base58 } from '../../util/base58';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { UserListItem } from '../abSpecific/UserListItem';
import { ButtonsModal } from '../modals/ButtonsModal';
import { ChallengeModal } from '../modals/ChallengeModal';
import { GradientFadeOut } from '../modals/GradientFadeout';
import { showQrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { MainButton } from '../themed/MainButton';
import { OutlinedTextInput } from '../themed/OutlinedTextInput';
import { ThemedScene } from '../themed/ThemedScene';
const MAX_DISPLAYED_LOCAL_USERS = 5;
export const PasswordLoginScene = (props) => {
    const { branding, route } = props;
    const { username, createAccountType = 'full' } = route.params;
    const { context, onComplete, onLogEvent = (event, values) => { } } = useImports();
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    const localUsers = useLocalUsers();
    const hasSavedUsers = localUsers.length > 0;
    const touch = useSelector(state => state.touch.type);
    const [passwordErrorMessage, setPasswordErrorMessage] = React.useState(undefined);
    const [usernameErrorMessage, setUsernameErrorMessage] = React.useState(undefined);
    const [password, setPassword] = React.useState('');
    const [showUsernameList, setShowUsernameList] = React.useState(false);
    const [dropdownY, setDropdownY] = React.useState(0);
    const [usernameItemHeight, setUsernameItemHeight] = React.useState(0);
    const [isScrollEnabled, setIsScrollEnabled] = React.useState(true);
    const [scrollViewHeight, setScrollViewHeight] = React.useState(0);
    const [contentHeight, setContentHeight] = React.useState(0);
    const passwordInputRef = React.useRef(null);
    const mDropContainerStyle = React.useMemo(() => {
        return Object.assign({ top: dropdownY }, styles.dropContainer);
    }, [styles, dropdownY]);
    const sAnimationMult = useSharedValue(0);
    const sScrollY = useSharedValue(0);
    const dFinalHeight = useDerivedValue(() => {
        return (usernameItemHeight *
            Math.min(localUsers.length, MAX_DISPLAYED_LOCAL_USERS));
    }, [usernameItemHeight, localUsers]);
    const scrollHandler = useAnimatedScrollHandler({
        onScroll: (e) => {
            sScrollY.value = e.contentOffset.y;
        }
    });
    // Gradually hide the bottom ScrollView gradient as the last item scrolls into
    // view
    const aGradientOpacity = useAnimatedStyle(() => {
        // Always hide the bottom ScrollView gradient if there's no entries below
        // the lower bound of the ScrollView
        if (MAX_DISPLAYED_LOCAL_USERS > localUsers.length)
            return { opacity: 0 };
        // Define the bounds at which the opacity should begin to change
        const minScroll = usernameItemHeight * (localUsers.length - MAX_DISPLAYED_LOCAL_USERS - 1);
        const maxScroll = usernameItemHeight * (localUsers.length - MAX_DISPLAYED_LOCAL_USERS);
        return {
            opacity: interpolate(sScrollY.value, [minScroll, maxScroll], [1, 0], Extrapolate.CLAMP)
        };
    });
    const aDropContainerStyle = useAnimatedStyle(() => ({
        height: dFinalHeight.value * sAnimationMult.value,
        opacity: showUsernameList
            ? withTiming(1, { duration: 50, easing: Easing.exp })
            : withTiming(0, { duration: 200, easing: Easing.exp })
    }), [showUsernameList]);
    const handleScrollViewLayout = (event) => {
        const { height: scrollViewHeight } = event.nativeEvent.layout;
        setScrollViewHeight(scrollViewHeight);
        setIsScrollEnabled(contentHeight > scrollViewHeight);
    };
    const handleContentLayout = (event) => {
        const { height: contentHeight } = event.nativeEvent.layout;
        setContentHeight(contentHeight);
        setIsScrollEnabled(contentHeight > scrollViewHeight);
    };
    const handleUsernameLayout = useHandler((event) => {
        setDropdownY(event.nativeEvent.layout.y + theme.rem(3.5));
    });
    const handleDropdownItemLayout = useHandler((event) => {
        if (event != null && usernameItemHeight === 0) {
            const { height } = event.nativeEvent.layout;
            setUsernameItemHeight(height);
        }
    });
    const handlePasswordChange = useHandler((password) => {
        setPasswordErrorMessage(undefined);
        setPassword(password);
    });
    const handleSubmit = useHandler((challengeId) => __awaiter(void 0, void 0, void 0, function* () {
        const otpAttempt = { type: 'password', username, password };
        try {
            Keyboard.dismiss();
            yield dispatch(login(otpAttempt, { challengeId }));
            onLogEvent('Pasword_Login');
        }
        catch (error) {
            const otpError = asMaybeOtpError(error);
            if (otpError != null) {
                dispatch({
                    type: 'NAVIGATE',
                    data: { name: 'otpError', params: { otpAttempt, otpError } }
                });
            }
            const usernameError = asMaybeUsernameError(error);
            if (usernameError != null) {
                setUsernameErrorMessage(s.strings.invalid_account);
                return;
            }
            const passwordError = asMaybePasswordError(error);
            if (passwordError != null) {
                setPasswordErrorMessage(s.strings.invalid_password);
                return;
            }
            const challengeError = asMaybeChallengeError === null || asMaybeChallengeError === void 0 ? void 0 : asMaybeChallengeError(error);
            if (challengeError != null) {
                const result = yield Airship.show(bridge => (React.createElement(ChallengeModal, { bridge: bridge, challengeError: challengeError })));
                if (result === true) {
                    // Try again with the passed challenge ID included
                    yield handleSubmit(challengeError.challengeId);
                }
                else {
                    setPasswordErrorMessage(s.strings.failed_captcha_error);
                }
                return;
            }
            console.warn('Unknown login error: ', error);
            setPasswordErrorMessage(error instanceof Error ? error.message : undefined);
        }
    }));
    const handleDelete = useHandler((userInfo) => {
        Keyboard.dismiss();
        Airship.show(bridge => {
            var _a;
            return (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.forget_account, message: sprintf(s.strings.forget_username_account, (_a = userInfo.username) !== null && _a !== void 0 ? _a : s.strings.username), buttons: {
                    ok: { label: s.strings.forget },
                    cancel: { label: s.strings.cancel, type: 'secondary' }
                } }));
        })
            .then((button) => __awaiter(void 0, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            if (context.forgetAccount != null) {
                yield context.forgetAccount(userInfo.loginId);
            }
            else {
                const { username } = userInfo;
                if (username == null)
                    throw new Error('No username');
                yield context.deleteLocalAccount(username);
            }
        }))
            .catch(showError);
    });
    const handleToggleUsernameList = useHandler(() => {
        // Dismiss the keyboard when opening the username list
        if (!showUsernameList)
            Keyboard.dismiss();
        setShowUsernameList(!showUsernameList);
    });
    const handleSelectUser = useHandler((userInfo) => {
        const { loginId, username } = userInfo;
        if (username == null) {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'pinLogin', params: { loginId } }
            });
        }
        else {
            handleChangeUsername(username);
            setShowUsernameList(false);
            const details = localUsers.find(info => info.username === username);
            if (details != null &&
                (details.pinLoginEnabled || (details.touchLoginEnabled && touch))) {
                dispatch({
                    type: 'NAVIGATE',
                    data: { name: 'pinLogin', params: { loginId } }
                });
            }
            else {
                if (passwordInputRef.current != null)
                    passwordInputRef.current.focus();
            }
        }
    });
    const handleChangeUsername = useHandler((username) => {
        setShowUsernameList(false);
        setPasswordErrorMessage(undefined);
        setUsernameErrorMessage(undefined);
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'passwordLogin', params: { username, createAccountType } }
        });
    });
    const handleSubmitRecoveryKey = useHandler((recoveryKey) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        if (((_a = base58.parseUnsafe(recoveryKey)) === null || _a === void 0 ? void 0 : _a.length) !== 32)
            return s.strings.recovery_token_invalid;
        dispatch(launchPasswordRecovery(recoveryKey));
        return true;
    }));
    const handleForgotPassword = useHandler(() => {
        Keyboard.dismiss();
        onLogEvent('Password_Login_Forgot_Password');
        Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: handleSubmitRecoveryKey, title: s.strings.password_recovery, message: s.strings.initiate_password_recovery, inputLabel: s.strings.recovery_token })));
    });
    const handleCreateAccount = useHandler(() => {
        onLogEvent('Password_Login_Create_Account');
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: hasSavedUsers || createAccountType === 'full'
                    ? 'newAccountUsername'
                    : 'newAccountPin',
                params: {}
            }
        });
    });
    const handleQrModal = useHandler(() => {
        Keyboard.dismiss();
        dispatch(showQrCodeModal());
    });
    const handleBack = useHandler(() => {
        if (onComplete != null)
            onComplete();
    });
    // The main hints dropdown animation depending on focus state of the
    // username dropdown
    React.useEffect(() => {
        sAnimationMult.value = withTiming(showUsernameList ? 1 : 0, {
            duration: 250,
            easing: Easing.inOut(Easing.circle)
        });
    }, [sAnimationMult, showUsernameList]);
    // Make scene scrollability react to keyboard visibility
    React.useEffect(() => {
        const keyboardDidShowListener = Keyboard.addListener('keyboardDidShow', (e) => {
            setIsScrollEnabled(contentHeight > scrollViewHeight - e.endCoordinates.height);
        });
        const keyboardDidHideListener = Keyboard.addListener('keyboardDidHide', () => {
            setIsScrollEnabled(contentHeight > scrollViewHeight);
        });
        return () => {
            // Cleanup the event listeners on unmount
            keyboardDidShowListener.remove();
            keyboardDidHideListener.remove();
        };
    }, [contentHeight, scrollViewHeight]);
    const renderUsername = () => {
        return (React.createElement(View, { style: styles.usernameWrapper },
            React.createElement(View, { style: styles.inputField, onLayout: handleUsernameLayout },
                React.createElement(OutlinedTextInput, { autoCorrect: false, autoFocus: true, clearIcon: !hasSavedUsers, error: usernameErrorMessage, label: s.strings.username, marginRem: [0.5, 1, 0.5, 1], returnKeyType: "next", testID: "usernameFormField", value: username, onChangeText: handleChangeUsername })),
            hasSavedUsers ? (React.createElement(TouchableOpacity, { testID: "userDropdownIcon", style: styles.dropdownButton, onPress: handleToggleUsernameList }, showUsernameList ? (React.createElement(MaterialIcon, { name: "expand-less", size: theme.rem(1.5), style: styles.iconColor })) : (React.createElement(MaterialIcon, { name: "expand-more", size: theme.rem(1.5), style: styles.iconColor })))) : null));
    };
    const renderDropdownList = () => {
        return (React.createElement(Animated.View, { style: [mDropContainerStyle, aDropContainerStyle] },
            React.createElement(Animated.ScrollView, { keyboardShouldPersistTaps: "always", scrollEventThrottle: 1, onScroll: scrollHandler }, localUsers.map(userInfo => {
                const { username } = userInfo;
                return (React.createElement(UserListItem, { key: username, userInfo: userInfo, onClick: handleSelectUser, onDelete: handleDelete, onLayout: handleDropdownItemLayout }));
            })),
            React.createElement(Animated.View, { style: aGradientOpacity },
                React.createElement(GradientFadeOut, null))));
    };
    const renderPassword = () => {
        return (React.createElement(View, { style: styles.inputField },
            React.createElement(OutlinedTextInput, { ref: passwordInputRef, autoCorrect: false, autoFocus: false, error: passwordErrorMessage, label: s.strings.password, marginRem: [0.5, 1, 0.5, 1], returnKeyType: "done", secureTextEntry: true, testID: "passwordFormField", value: password, onChangeText: handlePasswordChange, onSubmitEditing: handleSubmit })));
    };
    const renderButtons = () => {
        const buttonType = theme.preferPrimaryButton ? 'primary' : 'secondary';
        return (React.createElement(View, { style: styles.buttonsBox },
            React.createElement(MainButton, { type: "textOnly", onPress: handleForgotPassword, label: s.strings.forgot_password }),
            React.createElement(View, { style: styles.loginButtonBox },
                React.createElement(MainButton, { label: s.strings.login_button, testID: "loginButton", type: buttonType, disabled: username.length === 0 ||
                        password.length === 0 ||
                        usernameErrorMessage != null ||
                        passwordErrorMessage != null, onPress: handleSubmit })),
            React.createElement(MainButton, { type: "textOnly", testID: "createAccountButton", onPress: handleCreateAccount, label: s.strings.create_an_account }),
            React.createElement(TouchableOpacity, { onPress: handleQrModal },
                React.createElement(AntDesignIcon, { name: "qrcode", color: theme.icon, size: theme.rem(1.75) }))));
    };
    return (React.createElement(ThemedScene, { noUnderline: true, branding: branding, onBack: handleBack },
        React.createElement(KeyboardAwareScrollView, { style: styles.container, keyboardShouldPersistTaps: "handled", scrollEnabled: isScrollEnabled, onLayout: handleScrollViewLayout },
            React.createElement(View, { onLayout: handleContentLayout },
                React.createElement(LogoImageHeader, { branding: branding }),
                React.createElement(View, { style: styles.inputContainer },
                    renderUsername(),
                    renderDropdownList(),
                    renderPassword(),
                    renderButtons())))));
};
const getStyles = cacheStyles((theme) => ({
    dropContainer: {
        backgroundColor: theme.modal,
        borderRadius: theme.rem(0.5),
        zIndex: 1,
        borderColor: theme.iconTappable,
        borderWidth: theme.thinLineWidth,
        overflow: 'hidden',
        position: 'absolute',
        marginHorizontal: theme.rem(1),
        flex: 1
    },
    container: {
        flex: 1,
        paddingTop: theme.rem(2),
        paddingHorizontal: theme.rem(0.5)
    },
    inputContainer: {
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(2)
    },
    loginButtonBox: {
        marginVertical: theme.rem(0.25),
        width: '70%'
    },
    buttonsBox: {
        alignItems: 'center'
    },
    usernameWrapper: {
        flexDirection: 'row'
    },
    inputField: {
        flex: 1,
        marginBottom: theme.rem(1)
    },
    // TODO: Integrate dropdown into OutlinedTextInput
    dropdownButton: {
        justifyContent: 'center',
        alignItems: 'center',
        position: 'absolute',
        right: theme.rem(1.25),
        width: theme.rem(2.5),
        height: theme.rem(2),
        bottom: theme.rem(1.85)
    },
    iconColor: {
        color: theme.iconTappable
    }
}));
