import * as React from 'react';
import { ScrollView, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { checkAndRequestNotifications } from '../../../actions/LoginInitActions';
import s from '../../../common/locales/strings';
import { useHandler } from '../../../hooks/useHandler';
import { useImports } from '../../../hooks/useImports';
import { useDispatch } from '../../../types/ReduxTypes';
import { useTheme } from '../../services/ThemeContext';
import { AccountInfo } from '../../themed/AccountInfo';
import { EdgeText } from '../../themed/EdgeText';
import { FormError } from '../../themed/FormError';
import { MainButton } from '../../themed/MainButton';
import { ThemedScene } from '../../themed/ThemedScene';
const AccountReviewComponent = (props) => {
    const { username, password, pin, onNext } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    return (React.createElement(ThemedScene, { title: s.strings.review },
        React.createElement(ScrollView, { contentContainerStyle: styles.content },
            React.createElement(EdgeText, { style: styles.description, numberOfLines: 2 }, s.strings.tap_to_review_button),
            React.createElement(FormError, { marginRem: [0, 0, 2], title: s.strings.alert_dropdown_warning, numberOfLines: 2, isWarning: true }, s.strings.warning_message),
            React.createElement(AccountInfo, { marginRem: [0, 2.5], username: username, password: password, pin: pin }),
            React.createElement(View, { style: styles.actions },
                React.createElement(MainButton, { label: s.strings.create, type: "secondary", onPress: onNext })))));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(1.5)
    },
    description: {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(2)
    },
    actions: {
        flexDirection: 'row',
        justifyContent: 'center',
        marginTop: theme.rem(5),
        minHeight: theme.rem(3)
    }
}));
export const NewAccountReviewScene = (props) => {
    const { branding, route } = props;
    const dispatch = useDispatch();
    const { onLogin, onLogEvent = (event, values) => { } } = useImports();
    const handleNext = useHandler(() => {
        if (onLogin == null) {
            console.error('NewAccountReviewScene: onLogin not found');
            return;
        }
        onLogEvent(`Signup_Review_Done`);
        onLogin(route.params.account);
        dispatch(checkAndRequestNotifications(branding)).catch(error => console.log(error));
    });
    return React.createElement(AccountReviewComponent, Object.assign({}, route.params, { onNext: handleNext }));
};
export const UpgradeReviewScene = (props) => {
    const { branding, route } = props;
    const dispatch = useDispatch();
    const { onComplete, onLogEvent = (event, values) => { } } = useImports();
    const handleNext = useHandler(() => {
        onLogEvent(`Backup_Review_Done`);
        if (onComplete != null)
            onComplete();
        dispatch(checkAndRequestNotifications(branding)).catch(error => console.log(error));
    });
    return React.createElement(AccountReviewComponent, Object.assign({}, route.params, { onNext: handleNext }));
};
