import * as React from 'react';
import { TextInput, TouchableWithoutFeedback, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { fixSides, mapSides, sidesToMargin } from '../../util/sides';
import { useTheme } from '../services/ThemeContext';
import { PinDots } from './PinDots';
export const MAX_PIN_LENGTH = 4;
export const DigitInput = (props) => {
    const { pin, maxPinLength = MAX_PIN_LENGTH, marginRem, onChangePin } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const spacings = sidesToMargin(mapSides(fixSides(marginRem, 0.5), theme.rem));
    const inputRef = React.useRef(null);
    const handleRefocus = () => {
        if (inputRef.current != null)
            inputRef.current.focus();
    };
    return (React.createElement(TouchableWithoutFeedback, { onPress: handleRefocus },
        React.createElement(View, { style: [styles.container, spacings] },
            React.createElement(View, { style: styles.interactiveContainer },
                React.createElement(PinDots, { pinLength: pin.length, maxLength: maxPinLength })),
            React.createElement(TextInput, { ref: inputRef, style: styles.input, onChangeText: onChangePin, maxLength: maxPinLength, keyboardType: "number-pad", value: pin, autoFocus: true }))));
};
const getStyles = cacheStyles((theme) => ({
    container: {
        alignSelf: 'center'
    },
    interactiveContainer: {
        width: theme.rem(13)
    },
    input: {
        position: 'absolute',
        top: 0,
        left: 0,
        width: 0,
        height: 0,
        opacity: 0
    }
}));
