var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { forwardRef, useCallback, useEffect, useImperativeHandle, useRef, useState } from 'react';
import * as React from 'react';
import { ActivityIndicator, Platform, TextInput, TouchableOpacity, TouchableWithoutFeedback, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import Animated, { interpolateColor, useAnimatedStyle, useSharedValue, withDelay, withTiming } from 'react-native-reanimated';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import IonIcon from 'react-native-vector-icons/Ionicons';
import { fixSides, mapSides, sidesToMargin } from '../../util/sides';
import { useTheme } from '../services/ThemeContext.js';
// The ratio of the maxLength at which the remaining character count is shown
const MAX_LENGTH_WARNING_THRESHOLD = 0.8;
export const OutlinedTextInput = forwardRef((props, ref) => {
    const { 
    // Contents:
    error, valid, label, value, 
    // Appearance:
    clearIcon = true, marginRem, multiline = false, searchIcon = false, showSpinner = false, 
    // Callbacks:
    onBlur, onChangeText, onClear, onFocus, onHidePassword, 
    // TextInput:
    autoFocus = !searchIcon, blurOnClear = searchIcon, maxLength, secureTextEntry, testID, editableOnSpinner = false, onSubmitEditing } = props, inputProps = __rest(props, ["error", "valid", "label", "value", "clearIcon", "marginRem", "multiline", "searchIcon", "showSpinner", "onBlur", "onChangeText", "onClear", "onFocus", "onHidePassword", "autoFocus", "blurOnClear", "maxLength", "secureTextEntry", "testID", "editableOnSpinner", "onSubmitEditing"]);
    const theme = useTheme();
    const styles = getStyles(theme);
    const hasError = error != null;
    const hasValid = valid != null;
    const hasLabel = label != null;
    const hasValue = value !== '';
    // Show/Hide password input:
    const [hidePassword, setHidePassword] = React.useState(secureTextEntry !== null && secureTextEntry !== void 0 ? secureTextEntry : false);
    const handleHidePassword = () => setHidePassword(!hidePassword);
    // Imperative methods:
    const inputRef = useRef(null);
    function blur() {
        if (inputRef.current != null)
            inputRef.current.blur();
    }
    function clear() {
        if (inputRef.current != null)
            inputRef.current.clear();
        if (onChangeText != null)
            onChangeText('');
        if (blurOnClear)
            blur();
        if (onClear != null)
            onClear();
    }
    function focus() {
        if (inputRef.current != null)
            inputRef.current.focus();
    }
    function isFocused() {
        return inputRef.current != null ? inputRef.current.isFocused() : false;
    }
    useImperativeHandle(ref, () => ({ blur, clear, focus, isFocused }));
    // Captures the width of the placeholder label:
    const [labelWidth, setLabelWidth] = useState(0);
    const handleLabelLayout = (event) => setLabelWidth(event.nativeEvent.layout.width);
    // Captures the width of the counter label:
    const [counterWidth, setCounterWidth] = useState(0);
    const handleCounterLayout = (event) => setCounterWidth(event.nativeEvent.layout.width);
    // Animates between 0 and 1 based our error state:
    const subtextAnimation = useSharedValue(0);
    useEffect(() => {
        subtextAnimation.value = withTiming(hasError || hasValid ? 1 : 0);
    }, [subtextAnimation, hasError, hasValid]);
    // Animates between 0 and 1 based on focus:
    const baseDuration = 300;
    const focusAnimation = useSharedValue(0);
    // A delayed focus animation for translating the label up is required to
    // avoid overlapping into the opening animation of the top border.
    // A delayed focus animation is required for closing the top border when
    // animating everything back to their original positions.
    const animationDelay = 0.4 * baseDuration;
    const focusAnimationAlt = useSharedValue(0);
    const handleBlur = () => {
        focusAnimation.value = withDelay(animationDelay, withTiming(0, { duration: baseDuration }));
        focusAnimationAlt.value = withTiming(0, { duration: baseDuration });
        if (onBlur != null)
            onBlur();
    };
    const handleFocus = () => {
        focusAnimation.value = withTiming(1, { duration: baseDuration });
        focusAnimationAlt.value = withDelay(animationDelay, withTiming(1, { duration: baseDuration }));
        if (onFocus != null)
            onFocus();
    };
    const handleSubmitEditing = () => {
        if (onSubmitEditing != null)
            onSubmitEditing();
    };
    // Label dimensions:
    const labelLeft = theme.rem(1);
    const labelPadding = theme.rem(0.25); // Gap in the top line
    const labelShrink = 0.25; // How much to shrink the text
    const labelTranslateX = (searchIcon ? theme.rem(-1.875) : 0) +
        labelPadding +
        // Compensate for the scaling origin being in the center:
        -0.5 * labelShrink * labelWidth;
    const labelTranslateY = theme.rem(-1.5);
    // Counter dimensions
    const counterRight = theme.rem(1);
    const counterPadding = theme.rem(0.25); // Gap in the bottom line
    const counterTranslateX = counterPadding +
        // Compensate for the scaling origin being in the center:
        0.5 * counterWidth;
    // Pad right side of the bottom line when no counter is present,
    // to account for rounded corner.
    const cornerPadding = theme.rem(0.5);
    // React-controlled styles:
    const containerPadding = {
        paddingLeft: searchIcon ? theme.rem(2.875) : theme.rem(1),
        paddingRight: clearIcon ? theme.rem(2.875) : theme.rem(1)
    };
    const containerStyle = Object.assign(Object.assign(Object.assign({}, containerPadding), sidesToMargin(mapSides(fixSides(marginRem, 0.5), theme.rem))), { flexGrow: multiline ? 1 : 0, paddingVertical: multiline
            ? // Tweaked to align the first input line with the label text:
                Platform.OS === 'android'
                    ? theme.rem(0.75)
                    : theme.rem(0.625)
            : 0 });
    const textInputStyle = {
        flexGrow: multiline ? 1 : 0
    };
    // Animated styles:
    const getBorderColor = useCallback((errorValue, focusValue) => {
        'worklet';
        const interFocusColor = interpolateColor(focusValue, [0, 1], [
            theme.outlineTextInputBorderColor,
            theme.outlineTextInputBorderColorFocused
        ]);
        return interpolateColor(errorValue, [0, 1], [interFocusColor, hasError ? theme.dangerText : interFocusColor]);
    }, [theme, hasError]);
    const getLabelColor = useCallback((errorValue, focusValue) => {
        'worklet';
        const interFocusColor = interpolateColor(focusValue, [0, 1], [
            theme.outlineTextInputLabelColor,
            theme.outlineTextInputLabelColorFocused
        ]);
        return interpolateColor(errorValue, [0, 1], [interFocusColor, hasError ? theme.dangerText : interFocusColor]);
    }, [hasError, theme]);
    const bottomStyle = useAnimatedStyle(() => {
        const counterProgress = hasValue ? 1 : focusAnimation.value;
        return {
            borderColor: getBorderColor(subtextAnimation.value, focusAnimation.value),
            right: maxLength !== undefined &&
                value.length >= MAX_LENGTH_WARNING_THRESHOLD * maxLength
                ? counterRight +
                    counterProgress * (2 * counterPadding + counterWidth)
                : cornerPadding
        };
    });
    const leftStyle = useAnimatedStyle(() => ({
        borderColor: getBorderColor(subtextAnimation.value, focusAnimation.value)
    }));
    const rightStyle = useAnimatedStyle(() => ({
        borderColor: getBorderColor(subtextAnimation.value, focusAnimation.value)
    }));
    const topStyle = useAnimatedStyle(() => {
        const counterProgress = hasLabel
            ? hasValue
                ? 1
                : focusAnimation.value
            : 0;
        return {
            borderColor: getBorderColor(subtextAnimation.value, focusAnimation.value),
            left: labelLeft +
                counterProgress * (2 * labelPadding + labelWidth * (1 - labelShrink))
        };
    });
    const labelStyle = useAnimatedStyle(() => {
        const labelProgressAlt = hasValue ? 1 : focusAnimationAlt.value;
        return {
            color: getLabelColor(subtextAnimation.value, focusAnimation.value),
            transform: [
                { translateX: labelProgressAlt * labelTranslateX },
                { translateY: labelProgressAlt * labelTranslateY },
                { scale: 1 - labelProgressAlt * labelShrink }
            ]
        };
    });
    const counterStyle = useAnimatedStyle(() => {
        const counterProgress = hasValue ? 1 : focusAnimation.value;
        return {
            color: interpolateColor(subtextAnimation.value, [0, 1], [
                theme.secondaryText,
                hasError
                    ? theme.dangerText
                    : theme.outlineTextInputBorderColorFocused
            ]),
            opacity: counterProgress,
            transform: [
                { translateX: (1 - counterProgress) * counterTranslateX },
                { scale: counterProgress }
            ]
        };
    });
    const subtextStyle = useAnimatedStyle(() => ({
        opacity: subtextAnimation.value
    }));
    const showPasswordLineStyle = useAnimatedStyle(() => ({
        backgroundColor: getBorderColor(subtextAnimation.value, focusAnimation.value),
        transform: [
            { rotateZ: '45deg' },
            {
                scaleX: withTiming(hidePassword ? 1 : 0, { duration: baseDuration })
            }
        ]
    }));
    const AnimatedIonIcon = Animated.createAnimatedComponent(IonIcon);
    const eyeIconStyle = useAnimatedStyle(() => {
        return {
            color: getBorderColor(subtextAnimation.value, focusAnimation.value)
        };
    });
    // Character limit
    const charLimitLabel = maxLength === undefined ||
        value.length < MAX_LENGTH_WARNING_THRESHOLD * maxLength
        ? ''
        : `${maxLength - value.length}`;
    return (React.createElement(TouchableWithoutFeedback, { accessible: false, testID: testID, onPress: () => focus() },
        React.createElement(View, { style: [styles.container, containerStyle] },
            React.createElement(Animated.View, { style: [styles.bottomLine, bottomStyle] }),
            React.createElement(Animated.View, { style: [styles.leftCap, leftStyle] }),
            React.createElement(Animated.View, { style: [styles.rightCap, rightStyle] }),
            React.createElement(Animated.View, { style: [styles.topLine, topStyle] }),
            React.createElement(View, { style: [styles.labelContainer, containerPadding] },
                React.createElement(Animated.Text, { accessible: true, numberOfLines: 1, style: [styles.labelText, labelStyle], onLayout: handleLabelLayout, testID: `${testID}.labelText` }, label)),
            React.createElement(Animated.Text, { accessible: true, numberOfLines: 1, style: [
                    hasError ? styles.errorText : styles.validText,
                    subtextStyle
                ], testID: `${testID}.subText` }, error !== null && error !== void 0 ? error : valid),
            React.createElement(Animated.Text, { accessible: true, numberOfLines: 1, style: [styles.counterText, counterStyle], onLayout: handleCounterLayout, testID: `${testID}.charLimit` }, charLimitLabel),
            searchIcon ? (React.createElement(AntDesignIcon, { name: "search1", style: styles.searchIcon })) : null,
            clearIcon && hasValue && !showSpinner && !secureTextEntry ? (React.createElement(TouchableOpacity, { accessible: true, style: styles.clearTapArea, onPress: () => clear(), testID: `${testID}.clearIcon` },
                React.createElement(AntDesignIcon, { name: "close", style: styles.clearIcon }))) : null,
            showSpinner && !secureTextEntry ? (React.createElement(View, { style: styles.clearTapArea },
                React.createElement(ActivityIndicator, { style: styles.spinnerIcon }))) : null,
            secureTextEntry ? (React.createElement(TouchableWithoutFeedback, { testID: `${testID}.eyeIcon`, onPress: handleHidePassword },
                React.createElement(View, { style: styles.clearTapArea },
                    React.createElement(Animated.View, { style: [styles.eyeIconHideLine, showPasswordLineStyle] }),
                    React.createElement(AnimatedIonIcon, { accessible: true, name: "eye-outline", style: [styles.eyeIcon, eyeIconStyle] })))) : null,
            React.createElement(TextInput, Object.assign({ accessible: true, ref: inputRef }, inputProps, { autoFocus: autoFocus, multiline: multiline, editable: editableOnSpinner || !showSpinner, selectionColor: hasError ? theme.dangerText : theme.outlineTextInputTextColor, style: [styles.textInput, textInputStyle], testID: `${testID}.textInput`, textAlignVertical: "top", value: value, secureTextEntry: hidePassword, 
                // Callbacks:
                onBlur: handleBlur, onChangeText: onChangeText, onFocus: handleFocus, onSubmitEditing: handleSubmitEditing, maxLength: maxLength })))));
});
const getStyles = cacheStyles((theme) => {
    // A top or bottom line in the border puzzle:
    const commonLine = {
        borderTopWidth: theme.outlineTextInputBorderWidth,
        position: 'absolute',
        left: theme.rem(1),
        right: theme.rem(1)
    };
    // A left or right C-shape in the border puzzle:
    const commonCap = {
        borderBottomWidth: theme.outlineTextInputBorderWidth,
        borderTopWidth: theme.outlineTextInputBorderWidth,
        position: 'absolute',
        bottom: 0,
        top: 0,
        width: theme.rem(1)
    };
    // Common footer attributes, applies to the counter and the error text
    const footerCommon = {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.75),
        position: 'absolute'
    };
    const subtextCommon = Object.assign(Object.assign({}, footerCommon), { left: theme.rem(1.25), bottom: -theme.rem(1.25) });
    return {
        // Provides a layout container for the text input:
        container: {
            backgroundColor: theme.outlineTextInputColor,
            borderRadius: theme.rem(0.5),
            justifyContent: 'center',
            minHeight: theme.rem(3),
            paddingHorizontal: theme.rem(1)
        },
        // Provides a layout container for the placeholder label:
        labelContainer: {
            height: theme.rem(3),
            justifyContent: 'center',
            paddingHorizontal: theme.rem(1),
            position: 'absolute',
            left: 0,
            right: 0,
            top: 0
        },
        // The text input and placeholder label both float
        // in their respective containers, allowing React to center them:
        labelText: {
            alignSelf: 'flex-start',
            fontFamily: theme.fontFaceDefault,
            fontSize: theme.rem(1),
            padding: 0
        },
        textInput: {
            alignSelf: 'stretch',
            color: theme.outlineTextInputTextColor,
            fontFamily: theme.fontFaceDefault,
            fontSize: theme.rem(1),
            padding: 0
        },
        // We render our border in four pieces, so we can animate the top gap:
        bottomLine: Object.assign(Object.assign({}, commonLine), { bottom: 0 }),
        topLine: Object.assign(Object.assign({}, commonLine), { top: 0 }),
        leftCap: Object.assign(Object.assign({}, commonCap), { borderLeftWidth: theme.outlineTextInputBorderWidth, borderRightWidth: 0, borderBottomLeftRadius: theme.rem(0.5), borderTopLeftRadius: theme.rem(0.5), left: 0 }),
        rightCap: Object.assign(Object.assign({}, commonCap), { borderLeftWidth: 0, borderRightWidth: theme.outlineTextInputBorderWidth, borderBottomRightRadius: theme.rem(0.5), borderTopRightRadius: theme.rem(0.5), right: 0 }),
        // Icons:
        searchIcon: {
            color: theme.iconDeactivated,
            fontSize: theme.rem(1),
            padding: theme.rem(1),
            position: 'absolute',
            left: 0,
            top: 0
        },
        clearTapArea: {
            position: 'absolute',
            right: 0,
            top: 0
        },
        clearIcon: {
            color: theme.iconDeactivated,
            fontSize: theme.rem(1),
            padding: theme.rem(1)
        },
        spinnerIcon: {
            color: theme.icon,
            padding: theme.rem(1)
        },
        eyeIcon: {
            zIndex: 0,
            fontSize: theme.rem(1),
            padding: theme.rem(1)
        },
        eyeIconHideLine: {
            borderTopWidth: theme.thinLineWidth,
            borderTopColor: theme.modal,
            borderBottomColor: theme.modal,
            borderBottomWidth: theme.thinLineWidth,
            top: theme.rem(1.5) - theme.thinLineWidth,
            position: 'absolute',
            alignSelf: 'center',
            zIndex: 2,
            width: '40%',
            height: theme.thinLineWidth * 3
        },
        // The error text hangs out in the margin area below the main box:
        errorText: Object.assign(Object.assign({}, subtextCommon), { color: theme.dangerText }),
        validText: Object.assign(Object.assign({}, subtextCommon), { color: theme.outlineTextInputLabelColorFocused }),
        // The counter text splits the bottom right border line:
        counterText: Object.assign(Object.assign({}, footerCommon), { right: theme.rem(1.25), bottom: -theme.rem(0.45) })
    };
});
